@@ -316,6 +316,23 @@ impl<S: Integer> iter::FromIterator<S> for Vec4d<S> {
     }
 }
 
+impl<S: Integer> iter::Sum<Vec4d<S>> for Vec4d<S> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = Vec4d<S>>,
+    {
+        iter.fold(Vec4d::zero(), |sum, v| sum + v)
+    }
+}
+impl<'a, S: Integer> iter::Sum<&'a Vec4d<S>> for Vec4d<S> {
+    fn sum<I>(iter: I) -> Self
+    where
+        I: Iterator<Item = &'a Vec4d<S>>,
+    {
+        iter.fold(Vec4d::zero(), |sum, v| sum + v)
+    }
+}
+
 impl<S: Integer> fmt::Debug for Vec4d<S> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(f, "(")?;
@@ -690,6 +707,13 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_sum() {
+        let vs = vec![v4d(1, 0, -1, -2), v4d(0, 1, 2, 3), v4d(2, -3, 1, 0)];
+        assert_eq!(v4d(3, -2, 2, 1), vs.iter().map(|v: &Vec4d| v).sum());
+        assert_eq!(v4d(3, -2, 2, 1), vs.into_iter().map(|v: Vec4d| v).sum());
+    }
+
     #[test]
     fn test_debug() {
         assert_eq!("(2, -3, 5, -8)", format!("{:?}", v4d(2, -3, 5, -8)));
